package commandSystem;

import java.util.ArrayList;
import java.util.Collections;

import Events.Event;

public class EventQueue {
	private ArrayList<Integer> upQueue = new ArrayList<>();
	private ArrayList<Integer> downQueue = new ArrayList<>();
	private boolean emergencyState;

	private int currentFloor;
	private Direction direction;

	public EventQueue() {

		upQueue = new ArrayList<Integer>();
		downQueue = new ArrayList<Integer>();
		emergencyState = false;
		currentFloor = 0;
		direction = Direction.NONE;
	}

	/**
	 * Adds a new floor to <b>upQueue</b> or <b>downQueue</b> then sorts the
	 * elements.
	 * 
	 * @param queue    ArrayList<Int>
	 * @param floor    int - the floor we add to queue
	 * @param reversed boolean - reverse queue order
	 */
	public void appSort(ArrayList<Integer> queue, int floor, boolean reversed) {
		queue.add(floor);
		if (reversed)
			Collections.sort(queue, Collections.reverseOrder());
		else
			Collections.sort(queue);
	}

	/**
	 * Compute a request into the {@link #EventQueue} depending on the elevator
	 * status.
	 * 
	 * @param event - the request to compute
	 */
	public void computeEvent(Event event) {

		switch (event.getType()) {
		case CALLFROMFLOOR:
			if (emergencyState)
				return;
			switch (event.getRequestedDirection()) {
			case UP:
				if (!upQueue.contains(event.getIncomingCallFloor()))
					appSort(upQueue, event.getIncomingCallFloor(), false);
				break;
			case DOWN:
				if (!downQueue.contains(event.getIncomingCallFloor()))
					appSort(downQueue, event.getIncomingCallFloor(), true);
				break;
			default:
				break;
			}
			break;
		case CALLFROMELEVATOR:
			if (emergencyState)
				return;
			if (event.getRequestedFloor() > currentFloor && !upQueue.contains(event.getRequestedFloor()))
				appSort(upQueue, event.getRequestedFloor(), false);
			else if (event.getRequestedFloor() < currentFloor && !upQueue.contains(event.getRequestedFloor()))
				appSort(downQueue, event.getRequestedFloor(), true);
			break;
		case EMERGENCYSTOP:
			if (emergencyState)
				return;
			emergencyState = true;
			clearQueue();
			break;
		case CANCELEMERGENCYSTOP:
			emergencyState = false;
			break;
		case REACHEDFLOORSIGNAL:
			if (emergencyState)
				return;
			currentFloor = event.getCurrentFloor();
			direction = event.getCurrentDirection();
			break;
		default:
			break;
		}
	}

	/**
	 * Removes one instruction from the queue - has to be called when the elevator
	 * reaches targeted floor.
	 * 
	 * @param floor     the reached floor
	 * @param direction the direction of the elevator
	 */
	public void removeInstruction(int floor, Direction direction) {
		switch (direction) {
		case UP:
			if (upQueue.contains(floor))
				upQueue.remove(upQueue.indexOf(floor));
			else // we go up to the top of descending queue
				downQueue.remove(downQueue.indexOf(floor));
			break;
		case DOWN:
			if (downQueue.contains(floor))
				downQueue.remove(downQueue.indexOf(floor));
			else // we go down to the bottom of ascending queue
				upQueue.remove(upQueue.indexOf(floor));
			break;
		default:
			break;
		}
	}

	/**
	 * Returns the next floor the elevator has to go to
	 * 
	 * @return the next floor if it exists, else -1
	 */
	public int getNextInstruction() {
		int nextFloor = -1;
		switch (direction) {
		// get the first element of upQueue that is ABOVE the elevator's current floor
		case UP:
			for (int i = 0; i < upQueue.size() && nextFloor < 0; i++) {
				if (upQueue.get(i) >= currentFloor) {
					nextFloor = upQueue.get(i);
				}
			}
			// change direction if every floor in upQueue is UNDER the elevator's current
			// floor
			if (nextFloor < 0 && downQueue.size() > 0) {
				nextFloor = downQueue.get(0);
			}
			break;

		// get the first element of downQueue that is UNDER the elevator's current floor
		case DOWN:
			for (int i = 0; i < downQueue.size() && nextFloor < 0; i++) {
				if (downQueue.get(i) <= currentFloor) {
					nextFloor = downQueue.get(i);
				}
			}
			// change direction if every floor in downQueue is ABOVE the elevator's current
			// floor
			if (nextFloor < 0 && upQueue.size() > 0) {
				nextFloor = upQueue.get(0);
			}
			break;
		default:
			break;
		}
		return nextFloor;
	}

	public void clearQueue() {
		downQueue.removeAll(downQueue);
		upQueue.removeAll(upQueue);
	}
}
